@@ -1,6 +1,6 @@
 package DBIx::Custom;
 
-our $VERSION = '0.1662';
+our $VERSION = '0.1663';
 
 use 5.008001;
 use strict;
@@ -19,6 +19,8 @@ use DBIx::Custom::Tag;
 use DBIx::Custom::Util;
 use Encode qw/encode_utf8 decode_utf8/;
 
+our @COMMON_ARGS = qw/table query filter type/;
+
 __PACKAGE__->attr(
     [qw/data_source password pid user/],
     cache => 1,
@@ -238,8 +240,8 @@ sub dbh {
     }
 }
 
-our %VALID_DELETE_ARGS
-  = map { $_ => 1 } qw/table where append filter allow_delete_all query/;
+our %DELETE_ARGS
+  = map { $_ => 1 } @COMMON_ARGS, qw/where append allow_delete_all/;
 
 sub delete {
     my ($self, %args) = @_;
@@ -247,16 +249,15 @@ sub delete {
     # Check argument names
     foreach my $name (keys %args) {
         croak qq{Argument "$name" is invalid name}
-          unless $VALID_DELETE_ARGS{$name};
+          unless $DELETE_ARGS{$name};
     }
     
     # Arguments
-    my $table            = $args{table} || '';
+    my $table = $args{table} || '';
     croak qq{"table" option must be specified} unless $table;
-    my $where            = $args{where} || {};
-    my $append           = $args{append};
-    my $filter           = $args{filter};
-    my $allow_delete_all = $args{allow_delete_all};
+    my $where            = delete $args{where} || {};
+    my $append           = delete $args{append};
+    my $allow_delete_all = delete $args{allow_delete_all};
 
     # Where
     my $w;
@@ -295,16 +296,18 @@ sub delete {
     
     # Execute query
     my $ret_val = $self->execute(
-        $query, param  => $where, filter => $filter,
-        table => $table);
+        $query,
+        param  => $where,
+        table => $table,
+        %args
+    );
     
     return $ret_val;
 }
 
 sub delete_all { shift->delete(allow_delete_all => 1, @_) }
 
-our %VALID_DELETE_AT_ARGS
-  = map { $_ => 1 } qw/table where append filter query primary_key param/;
+our %DELETE_AT_ARGS = (%DELETE_ARGS, where => 1, primary_key => 1);
 
 sub delete_at {
     my ($self, %args) = @_;
@@ -312,7 +315,7 @@ sub delete_at {
     # Check argument names
     foreach my $name (keys %args) {
         croak qq{Argument "$name" is invalid name}
-          unless $VALID_DELETE_AT_ARGS{$name};
+          unless $DELETE_AT_ARGS{$name};
     }
     
     # Primary key
@@ -346,7 +349,7 @@ sub delete_at {
 
 sub DESTROY { }
 
-our %VALID_EXECUTE_ARGS = map { $_ => 1 } qw/param filter table/;
+our %EXECUTE_ARGS = map { $_ => 1 } @COMMON_ARGS, 'param';
 
 sub execute{
     my ($self, $query, %args)  = @_;
@@ -354,7 +357,7 @@ sub execute{
     # Check argument names
     foreach my $name (keys %args) {
         croak qq{Argument "$name" is invalid name}
-          unless $VALID_EXECUTE_ARGS{$name};
+          unless $EXECUTE_ARGS{$name};
     }
     
     my $params = $args{param} || {};
@@ -414,7 +417,7 @@ sub execute{
     }
     
     # Filter argument
-    my $f = DBIx::Custom::Util::array_filter_to_hash($args{filter})
+    my $f = DBIx::Custom::Util::array_to_hash($args{filter})
          || $query->filter || {};
     foreach my $column (keys %$f) {
         my $fname = $f->{$column};
@@ -430,13 +433,26 @@ sub execute{
     }
     $filter = {%$filter, %$f};
     
+    # Type
+    my $type = DBIx::Custom::Util::array_to_hash($args{type});
+    
     # Bind
-    my $bind = $self->_bind($params, $query->columns, $filter);
+    my $bind = $self->_bind($params, $query->columns, $filter, $type);
     
     # Execute
     my $sth = $query->sth;
     my $affected;
-    eval {$affected = $sth->execute(@$bind)};
+    eval {
+        for (my $i = 0; $i < @$bind; $i++) {
+            if (my $type = $bind->[$i]->{type}) {
+                $sth->bind_param($i + 1, $bind->[$i]->{value}, $type);
+            }
+            else {
+                $sth->bind_param($i + 1, $bind->[$i]->{value});
+            }
+        }
+        $affected = $sth->execute;
+    };
     $self->_croak($@, qq{. Following SQL is executed. "$query->{sql}"}) if $@;
     
     # Return resultset if select statement is executed
@@ -472,8 +488,7 @@ sub execute{
     return $affected;
 }
 
-our %VALID_INSERT_ARGS
-  = map { $_ => 1 } qw/table param append filter query/;
+our %INSERT_ARGS = map { $_ => 1 } @COMMON_ARGS, qw/param append/;
 
 sub insert {
     my ($self, %args) = @_;
@@ -481,15 +496,14 @@ sub insert {
     # Check argument names
     foreach my $name (keys %args) {
         croak qq{Argument "$name" is invalid name}
-          unless $VALID_INSERT_ARGS{$name};
+          unless $INSERT_ARGS{$name};
     }
     
     # Arguments
-    my $table  = $args{table};
+    my $table  = delete $args{table};
     croak qq{"table" option must be specified} unless $table;
-    my $param  = $args{param} || {};
-    my $append = $args{append} || '';
-    my $filter = $args{filter};
+    my $param  = delete $args{param} || {};
+    my $append = delete $args{append} || '';
     
     # Columns
     my @columns;
@@ -518,16 +532,14 @@ sub insert {
     my $ret_val = $self->execute(
         $query,
         param  => $param,
-        filter => $filter,
-        table => $table
+        table => $table,
+        %args
     );
     
     return $ret_val;
 }
 
-our %VALID_INSERT_AT_ARGS
-  = map { $_ => 1 } qw/table param where append filter
-                       query primary_key param/;
+our %INSERT_AT_ARGS = (%INSERT_ARGS, where => 1, primary_key => 1);
 
 sub insert_at {
     my ($self, %args) = @_;
@@ -535,7 +547,7 @@ sub insert_at {
     # Check argument names
     foreach my $name (keys %args) {
         croak qq{Argument "$name" is invalid name}
-          unless $VALID_INSERT_AT_ARGS{$name};
+          unless $INSERT_AT_ARGS{$name};
     }
     
     # Primary key
@@ -674,6 +686,8 @@ sub include_model {
         $table_alias = {%$table_alias, %{$model->table_alias}};
         
         # Table - Model
+        croak "Table name is duplicated"
+          if exists $self->{_model_from}->{$model->table};
         $self->{_model_from}->{$model->table} = $model->name;
     }
     
@@ -760,9 +774,9 @@ sub register_filter {
 
 sub register_tag { shift->query_builder->register_tag(@_) }
 
-our %VALID_SELECT_ARGS
-  = map { $_ => 1 } qw/table column where append relation filter query
-                       selection join/;
+our %SELECT_ARGS
+  = map { $_ => 1 } @COMMON_ARGS, qw/column where append relation
+                                     selection join/;
 
 sub select {
     my ($self, %args) = @_;
@@ -770,25 +784,25 @@ sub select {
     # Check argument names
     foreach my $name (keys %args) {
         croak qq{Argument "$name" is invalid name}
-          unless $VALID_SELECT_ARGS{$name};
+          unless $SELECT_ARGS{$name};
     }
     
     # Arguments
-    my $table = $args{table};
+    my $table = delete $args{table};
     my $tables = ref $table eq 'ARRAY' ? $table
                : defined $table ? [$table]
                : [];
-    my $columns   = $args{column};
-    my $selection = $args{selection} || '';
-    my $where     = $args{where} || {};
-    my $append    = $args{append};
-    my $filter    = $args{filter};
-    my $join =     $args{join} || [];
+    my $columns   = delete $args{column};
+    my $selection = delete $args{selection} || '';
+    my $where     = delete $args{where} || {};
+    my $append    = delete $args{append};
+    my $join      = delete $args{join} || [];
     croak qq{"join" must be array reference}
       unless ref $join eq 'ARRAY';
+    my $relation = delete $args{relation};
     
     # Add relation tables(DEPRECATED!);
-    $self->_add_relation_table($tables, $args{relation});
+    $self->_add_relation_table($tables, $relation);
     
     # SQL stack
     my @sql;
@@ -864,7 +878,7 @@ sub select {
     # Table
     unless ($selection) {
         push @sql, 'from';
-        if ($args{relation}) {
+        if ($relation) {
             my $found = {};
             foreach my $table (@$tables) {
                 push @sql, ($table, ',') unless $found->{$table};
@@ -909,7 +923,7 @@ sub select {
     push @sql, $swhere;
     
     # Relation(DEPRECATED!);
-    $self->_push_relation(\@sql, $tables, $args{relation}, $swhere eq '' ? 1 : 0);
+    $self->_push_relation(\@sql, $tables, $relation, $swhere eq '' ? 1 : 0);
     
     # Append statement
     push @sql, $append if $append;
@@ -923,15 +937,16 @@ sub select {
     
     # Execute query
     my $result = $self->execute(
-        $query, param  => $where, filter => $filter,
-        table => $tables);
+        $query,
+        param  => $where, 
+        table => $tables,
+        %args
+    );
     
     return $result;
 }
 
-our %VALID_SELECT_AT_ARGS
-  = map { $_ => 1 } qw/table column where append relation filter query selection
-                       param primary_key join/;
+our %SELECT_AT_ARGS = (%SELECT_ARGS, where => 1, primary_key => 1);
 
 sub select_at {
     my ($self, %args) = @_;
@@ -939,7 +954,7 @@ sub select_at {
     # Check argument names
     foreach my $name (keys %args) {
         croak qq{Argument "$name" is invalid name}
-          unless $VALID_SELECT_AT_ARGS{$name};
+          unless $SELECT_AT_ARGS{$name};
     }
     
     # Primary key
@@ -990,9 +1005,8 @@ sub setup_model {
     return $self;
 }
 
-our %VALID_UPDATE_ARGS
-  = map { $_ => 1 } qw/table param where append filter
-                       allow_update_all query/;
+our %UPDATE_ARGS
+  = map { $_ => 1 } @COMMON_ARGS, qw/param where append allow_update_all/;
 
 sub update {
     my ($self, %args) = @_;
@@ -1000,17 +1014,16 @@ sub update {
     # Check argument names
     foreach my $name (keys %args) {
         croak qq{Argument "$name" is invalid name}
-          unless $VALID_UPDATE_ARGS{$name};
+          unless $UPDATE_ARGS{$name};
     }
     
     # Arguments
-    my $table            = $args{table} || '';
+    my $table = delete $args{table} || '';
     croak qq{"table" option must be specified} unless $table;
-    my $param            = $args{param} || {};
-    my $where            = $args{where} || {};
-    my $append           = $args{append} || '';
-    my $filter           = $args{filter};
-    my $allow_update_all = $args{allow_update_all};
+    my $param            = delete $args{param} || {};
+    my $where            = delete $args{where} || {};
+    my $append           = delete $args{append} || '';
+    my $allow_update_all = delete $args{allow_update_all};
     
     # Update keys
     my @clumns = keys %$param;
@@ -1079,18 +1092,19 @@ sub update {
     return $query if $args{query};
     
     # Execute query
-    my $ret_val = $self->execute($query, param  => $param, 
-                                 filter => $filter,
-                                 table => $table);
+    my $ret_val = $self->execute(
+        $query,
+        param  => $param, 
+        table => $table,
+        %args
+    );
     
     return $ret_val;
 }
 
 sub update_all { shift->update(allow_update_all => 1, @_) };
 
-our %VALID_UPDATE_AT_ARGS
-  = map { $_ => 1 } qw/table param where append filter
-                       query primary_key param/;
+our %UPDATE_AT_ARGS = (%UPDATE_ARGS, where => 1, primary_key => 1);
 
 sub update_at {
     my ($self, %args) = @_;
@@ -1098,7 +1112,7 @@ sub update_at {
     # Check argument names
     foreach my $name (keys %args) {
         croak qq{Argument "$name" is invalid name}
-          unless $VALID_UPDATE_AT_ARGS{$name};
+          unless $UPDATE_AT_ARGS{$name};
     }
     
     # Primary key
@@ -1159,10 +1173,10 @@ sub where {
 }
 
 sub _bind {
-    my ($self, $params, $columns, $filter) = @_;
+    my ($self, $params, $columns, $filter, $type) = @_;
     
     # bind values
-    my @bind;
+    my $bind = [];
     
     # Build bind values
     my $count = {};
@@ -1192,13 +1206,17 @@ sub _bind {
         # Filter
         my $f = $filter->{$column} || $self->{default_out_filter} || '';
         
-        push @bind, $f ? $f->($value) : $value;
+        # Type
+        push @$bind, {
+            value => $f ? $f->($value) : $value,
+            type => $type->{$column}
+        };
         
         # Count up 
         $count->{$column}++;
     }
     
-    return \@bind;
+    return $bind;
 }
 
 sub _connect {
@@ -2399,6 +2417,17 @@ You can check SQL.
 
     my $sql = $query->sql;
 
+=item C<type> EXPERIMENTAL
+
+Specify database data type.
+
+    $dbi->select(type => [image => DBI::SQL_BLOB]);
+    $dbi->select(type => [[qw/image audio/] => DBI::SQL_BLOB]);
+
+This is used to bind paramter by C<bind_param()> of statment handle.
+
+    $sth->bind_param($pos, $value, DBI::SQL_BLOB);
+
 =back
 
 =head2 C<select_at()> EXPERIMENTAL
